# Generated by Django 5.2.5 on 2025-08-28 23:39

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [
        ("testapp", "0007_genericgroupobjectpermission"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="created_at",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]
