{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}

{% block object-tools-items %}
{% url opts|admin_urlname:'permissions' original.pk|admin_urlquote as history_url %}
<li><a href="{% add_preserved_filters history_url %}" class="permissionslink">{% trans "Object permissions" %}</a></li>
{{ block.super }}
{% endblock %}
